/* 
 *  Autor:    Erik Bartmann
 *  Internet: https://erik-bartmann.de/
 *  Datum:    17.07.2022 
 *  Version   1.01
*/

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels

// Declaration for SSD1306 display connected using software SPI (default case):
#define OLED_MOSI   9
#define OLED_CLK   10
#define OLED_DC    11
#define OLED_CS    12
#define OLED_RESET 13
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT,
  OLED_MOSI, OLED_CLK, OLED_DC, OLED_RESET, OLED_CS);

const char ADDR[] = {22, 24, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52};
const char DATA[] = {39, 41, 43, 45, 47, 49, 51, 53};
#define CLOCK 2 // Clock-Pin
#define RW    3 // Read/Write-Pin

void setup() {
  // Serielle Schnittstelle konfigurieren
  Serial.begin(19200); 
  // Adress-Pins als Input konfigurieren
  for (int pin = 0; pin < 16; pin++)
    pinMode(ADDR[pin], INPUT);
  // Daten-Pins als Input konfigurieren
  for (int pin = 0; pin < 8; pin++)
    pinMode(DATA[pin], INPUT);
  pinMode(CLOCK, INPUT); // Clock-Pin als Input
  pinMode(RW, INPUT);    // Read/Write als Input
  // Clock-Pin als Interrupt mit Funktion konfigurieren
  attachInterrupt(digitalPinToInterrupt(CLOCK), timerClock, RISING);

  if(!display.begin(SSD1306_SWITCHCAPVCC)) {
    Serial.println(F("SSD1306 allocation failed"));
    for(;;); // Don't proceed, loop forever
  }
  // Show initial display buffer contents on the screen --
  // the library initializes this with an Adafruit splash screen.
  display.display();
  delay(2000); // Pause for 2 seconds
  // Clear the buffer
  display.clearDisplay();
  drawTopic();
}

void drawTopic(void) {
  display.clearDisplay();
  display.setTextSize(2);      // Normal 1:1 pixel scale
  display.setTextColor(SSD1306_WHITE); // Draw white text
  //display.cp437(true);       // Use full 256 char 'Code Page 437' font
  display.setTextColor(SSD1306_WHITE); // Draw white text
  display.setCursor(0,0);              // Start at top-left corner
  display.println(F("Adr : Data"));
  display.display();
  delay(2000);
}

void drawBus(unsigned int addr, unsigned int data){
  display.setTextSize(3);      // Normal 1:1 pixel scale
  display.setTextColor(SSD1306_WHITE); // Draw white text
  char buffer[20];
  //display.cp437(true);         // Use full 256 char 'Code Page 437' font
  display.setTextColor(SSD1306_WHITE); // Draw white text
  display.setCursor(0,20);
  display.fillRect(0, 20, 128, 63, SSD1306_BLACK);
  sprintf(buffer, "%04X:%02X", addr, data);
  display.println(buffer);
  display.display();
}

void timerClock(){
  unsigned int address = 0; // Wert des Adressbusses
  unsigned int data= 0;     // Wert des Datenbusses
  char buffer[20];          // Ausgabepuffer für Formatierung
  for (int pin = 0; pin < 16; pin++) {
    int bit = digitalRead(ADDR[pin]) ? 1 : 0;
    Serial.print(bit);
    address = (address << 1) + bit;
  }
  Serial.print(" : ");
  for (int pin = 0; pin < 8; pin++) {
    int bit = digitalRead(DATA[pin]) ? 1 : 0;
    Serial.print(bit);
    data= (data << 1) + bit;
  }
  sprintf(buffer, "    $%04X : $%02X - %c", address, data, digitalRead(RW)?'R':'W');
  Serial.println(buffer);

  // Update OLED
  drawBus(address,data);
}

void loop() {/* leer */}
